#include "8255.h"

/*
General EUSART routine.
This code does not use interrupt.
Call "eusart_init()" periodically.
These routines does not stack, so FSR1 and FSR2 are NOT used.
The symbols "near" and "far" can be changed 
according to required speed and available resource.
*/

#define EUSART_SEND_BUFF_SIZE 8

#pragma udata 
static far unsigned char g_eusart_buff[EUSART_SEND_BUFF_SIZE];
#pragma udata access accessram
static near unsigned char g_temp1;
static near unsigned char g_eusart_buff_write, g_eusart_buff_read;

/* Structure of data sent from 18F14K50:
MSB XXXXXXXX LSB
    ||||||||
    |||||+++- indicates the bit position in 8255-PB
    |||||
    |++++---- corresponds to 8255-PA0-3 (between 0-9)
    |
    +-------- 0: key down; 1: key up

0x50-0x7F, 0xD0-0xFF can used for the other types of signals than key up/down.
(Currently, not assigned.  All registered.)

*/
extern unsigned char g_keymap[16];  // Defined in 8255.c
extern unsigned char g_key_pressed; // Defined in 8255.c
extern unsigned char g_8255PC; // Defined in 8255.c

void eusart_received(static near unsigned char s){
	// Write your own code here.
	// The signal from 18F14K50 corresponds to key press/relase (see above).
	// From this data, change the key map.
	static near unsigned char pos,mask,prev;
	static far unsigned char play_button=0;
	pos=s&0x78;
	pos>>=3;
	mask=s&0x07;
	mask=1<<mask;
	if (s&0x80) { // key release
		g_keymap[pos]|=mask;
		if (s==0xC2) { // Release SCROLL
			play_button=0;
		}
	} else if (s==prev) { // key press
		mask=~mask;
		g_keymap[pos]&=mask;
		if (s==0x42) { // press SCROLL
			if (!play_button){
				play_button=1;
				// Toggle CMT motor
				btoggle(g_8255PC,4);
			}
		}
	}
	prev=s;
	g_key_pressed=1;
}

void eusart_send(static unsigned char s){
	if (PIR1bits.TXIF && g_eusart_buff_write==g_eusart_buff_read) {
		TXREG=s;
	} else {
		g_eusart_buff[g_eusart_buff_write++]=s;
		if (g_eusart_buff_write==EUSART_SEND_BUFF_SIZE) g_eusart_buff_write=0;
	}	
}

void eusart_init(){
	// PIC initialization
	// EUSART settings at 2400 bps (Fosc = 64 MHz)
	TXSTA=0x26; //TX9=0, TXEN=1, SYNC=0, SENDB=0, BRGH=1, TRMT=1, TX9D=0
	BAUDCONbits.BRG16=1;
	// 19.2k baud was OK when CR (0.1 uF/330 ohm) as stress was used.
	// Current baud setting is 2400 bps, that is 10 times lower than above speed.
	SPBRGH=0x1A;
	SPBRG=0x0A;
	RCSTA=0x90; // SPEN=1, RX9=0, CREN=1, FERR=0, OERR=0, RX9D=0
	PIR1bits.RCIF=0;
	PIR1bits.TXIF=1;
	// Library intialization
	g_eusart_buff_read=0;
	g_eusart_buff_write=0;
}
void eusart_events(){
	if (PIR1bits.RCIF) {
		eusart_received(RCREG); // PIR1bits.RCIF is automatically cleared.
	}
	if (RCSTAbits.FERR) {
		g_temp1=RCREG;
		g_temp1=RCREG;
	}
	if (RCSTAbits.OERR) {
		RCSTAbits.CREN=0;
		RCSTAbits.CREN=1;
	}
	if (g_eusart_buff_write!=g_eusart_buff_read) {
		if (PIR1bits.TXIF) {
			TXREG=g_eusart_buff[g_eusart_buff_read++]; // PIR1bits.TXIF is automatically set.
			if (g_eusart_buff_read==EUSART_SEND_BUFF_SIZE) g_eusart_buff_read=0;
		}
	} 
}