#include "cg.h"

/* Configurations */
// 4xPLL, external clock input, use RA6 as output port
#pragma config PLLCFG = ON, FOSC = ECHPIO6, PRICLKEN = OFF, FCMEN = OFF, IESO = OFF
// Other features will be off
#pragma config PWRTEN = OFF, BOREN = OFF, BORV = 285, WDTEN = OFF, WDTPS = 1
// MCLRE enabled
#pragma config MCLRE = EXTMCLR
// MISC, not used this time
#pragma config CCP2MX = PORTB3, PBADEN = ON, CCP3MX = PORTC6
#pragma config HFOFST = ON, T3CMX = PORTB5, P2BMX = PORTC0
#pragma config STVREN = ON, LVP = ON, XINST = OFF, DEBUG = OFF
// Protectios will be all off.
#pragma config CP0 = OFF, CP1 = OFF, CP2 = OFF, CP3 = OFF, CPB = OFF, CPD = OFF
#pragma config WRT0 = OFF, WRT1 = OFF, WRT2 = OFF, WRT3 = OFF, WRTC = OFF, WRTB = OFF, WRTD = OFF
#pragma config EBTR0 = OFF, EBTR1 = OFF, EBTR2 = OFF, EBTR3 = OFF, EBTRB = OFF

/* global variables */
#pragma udata g_vram=0x200
unsigned char g_vram[0x400];
#pragma udata access accessram
unsigned near char g_cline, g_fsrl, g_fsrh;
unsigned near char g_PORTA,g_PORTB,g_PORTC;
unsigned near char g_temp0, g_temp1, g_temp2, g_temp3, g_temp4;
#pragma udata

/* CG-ROM emmbed in program memory */ 
#pragma romdata cgrom=0x3000
#include "cgrom.h"

/* Macros as the inline functions */
#define waitTMR0H(x) do { _asm MOVF TMR0L,W,ACCESS _endasm } while(TMR0H<x)
#define waitTMR0L(x) do {} while(TMR0L<x);
#define beginRaster() do {} while(INTCONbits.TMR0IF==0);\
	TMR0H=0xFC;\
	TMR0L+=0x07;\
	INTCONbits.TMR0IF=0
#define beginShow() _asm\
	MOVF POSTINC0, W, ACCESS\
	MOVWF TBLPTRL, ACCESS\
	TBLRD\
	MOVF POSTINC0, W, ACCESS\
	MOVWF TBLPTRL, ACCESS\
	MOVF TABLAT, W, ACCESS\
	TBLRD\
	_endasm
#define show8bits() _asm\
	MOVWF LATB, ACCESS\
	NOP\
	RLNCF LATB, F, ACCESS\
	NOP\
	RLNCF LATB, F, ACCESS\
	NOP\
	RLNCF LATB, F, ACCESS\
	NOP\
	RLNCF LATB, F, ACCESS\
	MOVF POSTINC0, W, ACCESS\
	RLNCF LATB, F, ACCESS\
	MOVWF TBLPTRL, ACCESS\
	RLNCF LATB, F, ACCESS\
	MOVF TABLAT, W, ACCESS\
	RLNCF LATB, F, ACCESS\
	TBLRD\
	_endasm

#pragma code hpInt=0x08
void hpInt(){
	/*
	RA0-RA3    A0-A3
	RB1-RB6    A4-A9
	RC0-RC7    D0-D7
	*/
_asm
	MOVF  PORTC,W,ACCESS
	MOVWF g_PORTC,ACCESS
	RRNCF PORTB,W,ACCESS
	MOVWF g_PORTB,ACCESS     // g_PORTB0-6 <-> A4-A9
	MOVF  PORTA,W,ACCESS
	ANDLW 0x0F
	MOVWF g_PORTA,ACCESS     // g_PORTA0-4 <-> A0-A3 only
	SWAPF g_PORTB,W,ACCESS   // W0-W1 <-> A8-A9, W4-W7 <-> A4-A7
	ANDLW 0x03               // W0-W1 <-> A8-A9 only
	ADDLW 0x02
	MOVWF FSR1H,ACCESS
	SWAPF g_PORTB,W,ACCESS   // W0-W1 <-> A8-A9, W4-W7 <-> A4-A7
	ANDLW 0xF0               // W4-W7 <-> A4-A7 only
	IORWF g_PORTA,W,ACCESS
	MOVWF FSR1L,ACCESS
	MOVF  g_PORTC,W,ACCESS
	MOVWF INDF1,ACCESS
	BCF INTCON, 1, ACCESS    // INTCONbits.INT0IF=0;
	BSF INTCON, 7, ACCESS    // Allow next interrupt here
	MOVF POSTINC0, W, ACCESS // Increment FSR0 (this may work in some case)
	RETFIE 1
_endasm
/*
	g_temp0=PORTA;
	g_temp1=PORTB;
	g_temp2=PORTC;
	g_temp3=FSR0H;
	g_temp4=FSR0L;
	FSR0L=g_temp0<<6;
	g_temp1&=0x3F;
	FSR0L|=g_temp1;
	g_temp0&=0x0C;
	FSR0H=g_temp0>>2;
	FSR0H+=0x02;
	INDF0=g_temp2;
	FSR0H=g_temp3;
	FSR0L=g_temp4;
	INTCONbits.INT0IF=0;
	_asm RETFIE 1 _endasm
*/
}

#pragma code
void show(){
	beginRaster();
	// 0.3 V until 1.5 usec	
	sync=1;
	video=0;
	do {} while (TMR0L & 2); 
	do {} while (TMR0L & 1); 
	waitTMR0L(0x1F);
	// 0 V until 6.2 usec
	sync=0;
	waitTMR0L(0x6A);
	// 0.3 V until 16 usec
	sync=1;

	// Register preparation for showing
	TBLPTRU=0;
	TBLPTRH=0x30|g_cline;

	// Wait until 16 usec (7 cycles earlier)
	waitTMR0H(0xFD);
	FSR0H=g_fsrh;
	FSR0L=g_fsrl;

	// Main showing routine follows
	beginShow();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	show8bits();
	video=0;
}

void blanc(){
	beginRaster();
	// 0.3 V until 1.5 usec	
	sync=1;
	video=0;
	do {} while (TMR0L & 2); 
	do {} while (TMR0L & 1); 
	waitTMR0L(0x1F);
	// 0 V until 6.2 usec
	sync=0;
	waitTMR0L(0x6A);
	// 0.3 V until end of raster
	sync=1;
}

void equalize(){
	beginRaster();
	// 0 V until 27.08 usec
	sync=0;
	video=0;
	waitTMR0H(0xFD);
	waitTMR0L(0xB8);
	// 0.3 V until 31.78
	sync=1;
	waitTMR0H(0xFE);
	waitTMR0L(0x03);
	// 0 V until 58.76 usec
	sync=0;
	waitTMR0H(0xFF);
	waitTMR0L(0xB3);
	// 0.3 v until end of raster
	sync=1;
}
void preEqualize(){
	beginRaster();
	// 0 V until 2.3 usec
	sync=0;
	video=0;
	waitTMR0L(0x2C);
	// 0.3 V until 31.78 usec
	sync=1;
	waitTMR0H(0xFE);
	waitTMR0L(0x03);
	// 0 v until 34.08 usec
	sync=0;
	waitTMR0L(0x28);
	// 0.3 V until end of raster
	sync=1;
}
#define postEqualize() preEqualize()

/*
RE3 (MCLR) Reset input
RB0 (INT0) chip enable (and write)
RB1-RB6    A4-A9
RA0-RA3    A0-A3
RC0-RC7    D0-D7
RB7        video (WR for ROM generation)
RA4        Hsync (A10 for ROM generation)
RA5        Vsync (A11) for ROM generation)
RA6        Reset output
RA7 (CLKI) clock input
*/

#pragma udata access accessram
void main(){
	static near unsigned char i;

	// prepare test pattern
	i=0;
	do{
		g_vram[i]=i;
		g_vram[0x100+i]=i;
		g_vram[0x200+i]=i;
		g_vram[0x300+i]=i;
	} while((++i));

	// Copy 4 Kb monitor ROM to RAM
	monitorROM();

	// Timer0 on
	T0CON=0x88;
	// RA0-3: input, RA4-6: output, RA7: input
	ANSELA=0x00;
	TRISA=0x8F;
	// RB0-6: input, RB7: output
	ANSELB=0x00;
	TRISB=0x7F;
	// RC0-7: all input;
	ANSELC=0x00;
	TRISC=0xFF;
	// interrupt on RB0 falls
	RCONbits.IPEN=0;
	INTCON2bits.INTEDG0=0;
	INTCON=0x90;
	// Pull up on port B
	WPUB=0xFF;
	INTCON2bits.RBPU=0;

	// Terminate reset signal
	LATAbits.LATA6=1;

	// main loop
	do {
		preEqualize();
		preEqualize();
		preEqualize();
		equalize();
		equalize();
		equalize();
		postEqualize();
		postEqualize();
		postEqualize();
		busy=0; // Pseudo V sync
		i=11;
		do {
			blanc();
		} while (--i);
		busy=1; // Pseudo V sync ends here

		// Visible area
		i=21;
		do {
			blanc();
		} while (--i);
		i=25;
		g_fsrh=0x02;
		g_fsrl=0x00;
		do {
			g_cline=0;
			do {show();} while ((++g_cline)<8);
			if (215<g_fsrl) g_fsrh++;
			g_fsrl+=40;
		} while (--i);
		i=21;
		do {
			blanc();
		} while (--i);
	} while(1);
}