#include "kmz80usb.h"

/*
	General debug routine.
	Note that at least 256 bytes of EEPROM is required to use these functions.
	"debug_init()" bust be executed at the initiation.
	Use either "debug_write()" or "debug_write_async()" to leave byte messages.
	Also call "debug_flush()" if "debug_write_async()" is used.
*/

#define DEBUG_EEADR EEADR
#define DEBUG_EEDATA EEDATA
#define DEBUG_EECON1 EECON1
#define DEBUG_EECON2 EECON2
#define DEBUG_WR EECON1bits.WR
#define DEBUG_RD EECON1bits.RD
#define DEBUG_WREN EECON1bits.WREN
#define DEBUG_EEIF PIR2bits.EEIF
#define DEBUG_BUFF_SIZE 64
#define DEBUG_START_ADDR 0x80

/* General EEPROM writing routine */
void debug_eeprom_write(unsigned char addr, unsigned char s){
	DEBUG_WREN=0;
	DEBUG_EEIF=0;
	DEBUG_EEADR=addr;
	DEBUG_EEDATA=s;
	DEBUG_EECON1=0;
	DEBUG_WREN=1;
	DEBUG_EECON2=0x55;
	DEBUG_EECON2=0xAA;
	DEBUG_WR=1;
}

/* General EEPROM reading routine */
unsigned char debug_eeprom_read(unsigned char addr){
	DEBUG_EEADR=addr;
	DEBUG_RD=1;
	return DEBUG_EEDATA;
}

void debug_init(){
	unsigned char addr;
	addr=0;
	do{
		if (debug_eeprom_read(addr)!=0xFF) {
			debug_eeprom_write(addr,0xFF);
			while(!DEBUG_EEIF);
		}
	} while(++addr);
	DEBUG_EEIF=1; // for debug_write_async (this is redundant, I know.)
}


static unsigned char g_buff[DEBUG_BUFF_SIZE];
static unsigned char g_buff_write=0,g_buff_read=0;

void debug_write_async(unsigned char s){
	static unsigned char addr=DEBUG_START_ADDR;
	if (DEBUG_EEIF) { // Buffer is empty.
		debug_eeprom_write(addr,s);
		addr++;
	} else { // There is already something to write to EEPROM.
		g_buff[g_buff_write++]=s;
		if (DEBUG_BUFF_SIZE<=g_buff_write) g_buff_write=0;
	}
}

void debug_flush(){
	if (!DEBUG_EEIF) return;
	if (g_buff_read==g_buff_write) return;
	debug_write_async(g_buff[g_buff_read++]);
	if (DEBUG_BUFF_SIZE<=g_buff_read) g_buff_read=0;
}

void debug_write(unsigned char s){
	debug_write_async(s);
	while(!DEBUG_EEIF);
}
