#include "kmz80usb.h"

/*
General EUSART routine.
This code does not use interrupt.
Call "eusart_init()" periodically.
These routines does not stack, so FSR1 and FSR2 are NOT used.
The symbols "near" and "far" can be changed 
according to required speed and available resource.
*/

#define EUSART_SEND_BUFF_SIZE 8

#pragma udata
extern unsigned char g_kana_cmt; // defined in kmz80usb.c
static far unsigned char g_eusart_buff[EUSART_SEND_BUFF_SIZE];
#pragma udata access accessram
static near unsigned char g_temp1;
static near unsigned char g_eusart_buff_write, g_eusart_buff_read;

void eusart_received(static near unsigned char s){
	// Write your own code here.
	static near unsigned char cache;
	if (s==cache) return;
	cache=s;
	switch(s&0xF0){
		case 0x00: // receiving CMT motor and caps status
			g_kana_cmt=s;
			break;
		default:
			break;
	}
}

void eusart_send(static near unsigned char s){
	if (PIR1bits.TXIF && g_eusart_buff_write==g_eusart_buff_read) {
		TXREG=s;
	} else {
		g_eusart_buff[g_eusart_buff_write++]=s;
		if (g_eusart_buff_write==EUSART_SEND_BUFF_SIZE) g_eusart_buff_write=0;
	}	
}

void eusart_init(){
	// PIC initialization
	// EUSART settings at 2400 bps (Fosc = 48 MHz)
	TXSTA=0x26; //TX9=0, TXEN=1, SYNC=0, SENDB=0, BRGH=1, TRMT=1, TX9D=0
	BAUDCONbits.BRG16=1;
	// 19.2k baud was OK when CR (0.1 uF/330 ohm) as stress was used.
	// Current baud setting is 2400 bps, that is 10 times lower than above speed.
	SPBRGH=0x13;
	SPBRG=0x87;
	RCSTA=0x90; // SPEN=1, RX9=0, CREN=1, FERR=0, OERR=0, RX9D=0
	PIR1bits.RCIF=0;
	PIR1bits.TXIF=1;
	// Iibrary intialization
	g_eusart_buff_read=0;
	g_eusart_buff_write=0;
}
void eusart_events(){
	if (PIR1bits.RCIF) {
		eusart_received(RCREG); // PIR1bits.RCIF is automatically cleared.
	}
	if (RCSTAbits.FERR) {
		g_temp1=RCREG;
		g_temp1=RCREG;
	}
	if (RCSTAbits.OERR) {
		RCSTAbits.CREN=0;
		RCSTAbits.CREN=1;
	}
	if (g_eusart_buff_write!=g_eusart_buff_read) {
		if (PIR1bits.TXIF) {
			TXREG=g_eusart_buff[g_eusart_buff_read++]; // PIR1bits.TXIF is automatically set.
			if (g_eusart_buff_read==EUSART_SEND_BUFF_SIZE) g_eusart_buff_read=0;
		}
	} 
}