#include "kmz80usb.h"
#include "charcode.h"


#define ps2_clock_out TRISCbits.TRISC5
#define ps2_data_out  TRISCbits.TRISC3
#define ps2_clock_in  PORTCbits.RC5
#define ps2_data_in   PORTCbits.RC3
#define ps2_clock_lat LATCbits.LATC5
#define ps2_data_lat  LATCbits.LATC3

#define PS2IDLE  0
#define PS2READ0 10
#define PS2READ1 11
#define PS2READ2 12
#define PS2READ3 13
#define PS2READ4 14
#define PS2READ5 15
#define PS2READ6 16
#define PS2READ7 17
#define PS2READ8 18
#define PS2READ9 19

#pragma udata access accessram
unsigned near char g_ps2_status;
unsigned near char g_ps2_prev_clock;
unsigned near char g_ps2_receive_char;
unsigned near char g_ps2_parity;
static unsigned near char g_temp1;
#pragma udata



//#define rotateL(x) _asm RLNCF x,F,ACCESS _endasm
#define rotateR(x) _asm RRNCF x,F,ACCESS _endasm

void ps2_received(static unsigned char data){
	static unsigned char breakcode=0;
	if (data==0xF0) {
		breakcode=1;
	} else if (0<data && data<0x90) {
		data=charcode[data];
		if (breakcode) data=data|0x80;
		breakcode=0;
		// Serial sending routine
		// Send the same character twice
		eusart_send(data);
		eusart_send(data);
	} else {
		breakcode=0;
	}
}
void ps2_error(){
	// Currently, do nothing.
}

void ps2_init(){
	// IDLE mode
	ps2_clock_lat=0;
	ps2_data_lat=0;
	ps2_clock_out=1;
	ps2_data_out=1;
	g_ps2_status=PS2IDLE;
}

void ps2_receive(){
	if (PS2READ9<g_ps2_status) return; // Not reading mode.
	ps2_clock_out=1;
	ps2_data_out=1;
	if (ps2_clock_in==1) {
		g_ps2_prev_clock=1;
		return;
	}
	if (g_ps2_prev_clock==0) return;
	g_ps2_prev_clock=0;
	switch(g_ps2_status){
		case PS2IDLE: // Start receiving data
			if (ps2_data_in==0) { // First bit must be zero.
				g_ps2_receive_char=0;
				g_ps2_parity=0xFF;
				g_ps2_status=PS2READ0;
			} else { // If not zero, error occured.
				g_ps2_status=PS2IDLE;
				ps2_error();
			}
			break;
		case PS2READ0: case PS2READ1: case PS2READ2: case PS2READ3:
		case PS2READ4: case PS2READ5: case PS2READ6: case PS2READ7:
			if (ps2_data_in) {
				g_ps2_receive_char++;
				g_ps2_parity=!g_ps2_parity;
			}
			rotateR(g_ps2_receive_char);
			g_ps2_status++;
			break;
		case PS2READ8:
			if (g_ps2_parity && ps2_data_in || (!g_ps2_parity) && (!ps2_data_in)) {
				g_ps2_status++;
			} else {
				g_ps2_status=PS2IDLE;
				ps2_error();
			}
			break;
		case PS2READ9:
			if (ps2_data_in) {
				if (g_ps2_receive_char==0xFE) { // Resend request
					g_ps2_status=PS2IDLE;
					ps2_error();
				} else {
					g_ps2_status=PS2IDLE;
					ps2_received(g_ps2_receive_char);
				}
			} else {
				g_ps2_status=PS2IDLE;
				ps2_error();
			}
			break;
		default:
			break;
	}
}

